#colours
white = 'white'

# Graphviz colour scheme: paired12
(color1, color2, color3, color4, color5, color6, color7, color8, color9, color10, color11, color12) = \
    ("#a6cee3","#1f78b4", "#b2df8a", "#33a02c", "#fb9a99", "#e31a1c", "#fdbf6f", "#ff7f00", "#cab2d6", "#6a3d9a", "#ffff99", "#b15928" )

# Graphviz colour scheme: grey6
(grey1, grey2, grey3, grey4, grey5, grey6) = \
    ("#f7f7f7", "#d9d9d9", "#bdbdbd", "#969696", "#636363", "#252525" )

#attributes
(color, fillcolor, fontcolor, label, style, width) = \
    "color fillcolor fontcolor label style width".split(' ')


